/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */
package com.microej.demo.thermorest;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;
import java.util.Set;

import com.microej.demo.thermorest.cloud.CloudConnector;
import com.microej.demo.thermorest.endpoint.ThermostatEndPoint;
import com.microej.demo.thermorest.model.ThermostatModel;

import ej.basictool.ArrayTools;
import ej.components.dependencyinjection.ServiceLoader;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.hoka.http.HTTPRequest;
import ej.hoka.http.HTTPResponse;
import ej.hoka.http.body.StringBodyParserFactory;
import ej.restserver.RestEndpoint;
import ej.restserver.RestServer;
import ej.restserver.endpoint.AliasEndpoint;
import ej.restserver.endpoint.GzipResourceEndpoint;
import ej.restserver.endpoint.ResourceRestEndpoint;

/**
 * A thermostat using a HTTP server providing a web view to manage the a thermostat.
 */
public class HeadlessThermostat extends Observable implements Observer {

	private static final int JOB_COUNT_BY_SESSION = 1;
	private static final int MAX_SIMULTANEOUS_CONNECTION = 5;

	private static final int SERVER_PORT = 80;

	private static final String SLASH = "/"; //$NON-NLS-1$
	private static final String HOME_RESOURCE = SLASH + "index.html"; //$NON-NLS-1$
	private static final String SERVER_RESOURCES_PATH = "/com/microej/demo/thermorest/"; //$NON-NLS-1$
	private static final String SERVER_STATIC_ENDPOINT_FILENAME = "static.resources.list"; //$NON-NLS-1$

	private RestServer server;
	private final ThermostatModel model;
	private int port = SERVER_PORT;

	private CloudConnector[] cloudConnectors;
	private final CloudObservable cloudObservable;

	private RestEndpoint[] endpoints = new RestEndpoint[0];

	private boolean isLogged;

	/**
	 * Instantiates an {@link HeadlessThermostat} with a ThermostatModel available by the {@link ServiceLoader}.
	 */
	public HeadlessThermostat() {
		this(ServiceLoaderFactory.getServiceLoader().getService(ThermostatModel.class));
	}

	/**
	 * Instantiates a {@link HeadlessThermostat} with a specific {@link ThermostatModel}.
	 *
	 * @param model
	 *            the {@link ThermostatModel} to use.
	 */
	public HeadlessThermostat(ThermostatModel model) {
		this.model = model;
		this.cloudConnectors = new CloudConnector[0];
		this.cloudObservable = new CloudObservable();
	}

	/**
	 * Sets the server port.
	 *
	 * @param port
	 *            the server port to set.
	 */
	public void setPort(int port) {
		this.port = port;
	}

	/**
	 * Gets the server port.
	 *
	 * @return the server port.
	 */
	public int getPort() {
		return this.port;
	}

	/**
	 * Starts the thermostat server.
	 *
	 * @throws IOException
	 *             when the server could not be started.
	 */
	public void start() throws IOException {
		this.server = new RestServer(this.port, MAX_SIMULTANEOUS_CONNECTION, JOB_COUNT_BY_SESSION);
		this.server.setBodyParserFactory(new StringBodyParserFactory());
		RestEndpoint[] endpoints = this.endpoints;
		for (RestEndpoint restEndpoint : endpoints) {
			this.server.addEndpoint(restEndpoint);
		}
		this.server.addEndpoint(new ThermostatEndPoint(this));
		try (InputStream staticFiles = HeadlessThermostat.class
				.getResourceAsStream(SERVER_RESOURCES_PATH + SERVER_STATIC_ENDPOINT_FILENAME)) {
			createStaticEndpoints(staticFiles, HOME_RESOURCE, SERVER_RESOURCES_PATH);
		}
		this.server.start();
	}

	/**
	 * Stops the server.
	 */
	public void stop() {
		RestServer restServer = this.server;
		if (restServer != null) {
			restServer.stop();
			this.server = null;
		}
	}

	/**
	 * Gets the associated model.
	 *
	 * @return the model.
	 */
	public ThermostatModel getModel() {
		return this.model;
	}

	/**
	 * Adds a cloud connector for the {@link HeadlessThermostat}.
	 *
	 * @param connector
	 *            the connector to add.
	 */
	public void addCloudConnector(CloudConnector connector) {
		connector.addObserver(this);
		this.cloudConnectors = ArrayTools.add(this.cloudConnectors, connector);
		this.model.addListener(connector);
		change(connector);
	}

	/**
	 * Removes a cloud connector.
	 *
	 * @param connector
	 *            the connector to remove.
	 */
	public void removeCloudConnector(CloudConnector connector) {
		connector.deleteObserver(this);
		this.cloudConnectors = ArrayTools.remove(this.cloudConnectors, connector);
		this.model.removeListener(connector);
		change(connector);
	}

	/**
	 * Gets the cloud connectors.
	 *
	 * @return the cloud connectors.
	 */
	public CloudConnector[] getCloudConnectors() {
		return this.cloudConnectors.clone();
	}

	/**
	 * Add an observer to be notified when a {@link CloudConnector} is added, removed or changed.
	 *
	 * @param observer
	 *            the observer to be add.
	 */
	public void addCloudConnectorObserver(Observer observer) {
		this.cloudObservable.addObserver(observer);
	}

	/**
	 * Remove an observer notified when a {@link CloudConnector} is added, removed or changed.
	 *
	 * @param observer
	 *            the observer to be removed.
	 */
	public void removeCloudConnectorObserver(Observer observer) {
		this.cloudObservable.deleteObserver(observer);
	}

	@Override
	public void update(Observable o, Object arg) {
		change(arg);
	}

	/**
	 * Checks if a client has been logged.
	 *
	 * @return <code>true</code> if a client has been logged into the Server.
	 */
	public boolean clientIsLogged() {
		return this.isLogged;
	}

	/**
	 * Gets the additional endpoints.
	 *
	 * @return the endpoints.
	 */
	public RestEndpoint[] getEndpoints() {
		return this.endpoints;
	}

	/**
	 * Sets the additional endpoints.
	 *
	 * @param endpoints
	 *            the endpoints to set, cannot be <code>null</code>.
	 */
	public void setEndpoints(RestEndpoint... endpoints) {
		if (endpoints == null) {
			throw new NullPointerException();
		}
		this.endpoints = endpoints;
	}

	private void change(Object connector) {
		this.cloudObservable.setChanged();
		this.cloudObservable.notifyObservers(connector);
	}

	private void setClientIsLogged(boolean b) {
		if (this.isLogged != b) {
			this.isLogged = b;
			setChanged();
			notifyObservers(this);
		}
	}

	private void createStaticEndpoints(InputStream resourceFile, String homePage, String baseResourceDir)
			throws IOException {
		Properties filesProperties = new Properties();
		filesProperties.load(resourceFile);
		Set<String> files = filesProperties.stringPropertyNames();
		for (String filePath : files) {
			filePath = filePath.trim();
			String endpoint = filePath;
			if (filePath.startsWith(baseResourceDir)) {
				endpoint = SLASH + filePath.substring(baseResourceDir.length());
			}
			RestEndpoint restEndpoint;
			if (filePath.endsWith(GzipResourceEndpoint.GZIP_FILE_EXTENSION)) {
				endpoint = endpoint.substring(0, endpoint.length() - GzipResourceEndpoint.GZIP_FILE_EXTENSION.length());
				restEndpoint = new GzipResourceEndpoint(endpoint, filePath);
			} else {
				restEndpoint = new ResourceRestEndpoint(endpoint, filePath);
			}

			addEndpoint(restEndpoint);
			if (endpoint.equals(homePage)) {
				addEndpoint(new AliasEndpoint(SLASH, restEndpoint));
			}
		}
	}

	/**
	 * Overrides the endpoint to sets the {@link HeadlessThermostat} to be logged.
	 *
	 * @param endpoint
	 *            the endpoint to add.
	 */
	private void addEndpoint(final RestEndpoint endpoint) {
		this.server.addEndpoint(new RestEndpoint(endpoint.getURI()) {

			@Override
			public HTTPResponse get(HTTPRequest request, Map<String, String> headers, Map<String, String> parameters) {
				setClientIsLogged(true);
				return endpoint.get(request, headers, parameters);
			}

			@Override
			public HTTPResponse delete(HTTPRequest request, Map<String, String> headers,
					Map<String, String> parameters) {
				setClientIsLogged(true);
				return endpoint.delete(request, headers, parameters);
			}

			@Override
			public HTTPResponse post(HTTPRequest request, Map<String, String> headers, Map<String, String> parameters) {
				setClientIsLogged(true);
				return endpoint.post(request, headers, parameters);
			}

			@Override
			public HTTPResponse put(HTTPRequest request, Map<String, String> headers, Map<String, String> parameters) {
				setClientIsLogged(true);
				return endpoint.put(request, headers, parameters);
			}
		});
	}

	private static class CloudObservable extends Observable {
		@Override
		public void setChanged() {
			super.setChanged();
		}
	}

}
