/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */
package com.microej.demo.thermorest.cloud.aws;

import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.Properties;

import javax.net.ssl.SSLContext;

import com.microej.demo.thermorest.Messages;
import com.microej.demo.thermorest.cloud.PeriodicPublisherCloud;

import ej.aws.iot.Thing;
import ej.aws.iot.pubsub.ConnectOptions;
import ej.aws.iot.pubsub.IotCoreException;
import ej.aws.iot.pubsub.TopicListener;
import ej.util.message.Level;

/**
 * A publisher using Amazon AWS could to publish its data.
 */
public class ThermostatAWS extends PeriodicPublisherCloud implements TopicListener {

	private static final String AWS = "aws"; //$NON-NLS-1$
	private static final String BROKER = ".broker."; //$NON-NLS-1$
	private static final String NAME = AWS.toUpperCase();

	/**
	 * Property to get the path to the aws broker. (e.g. {@value #AWS_BROKER_URL}=mydomain.amazonaws.com).
	 */
	public static final String AWS_BROKER_URL = AWS + BROKER + "url"; //$NON-NLS-1$

	/**
	 * Property to get the path to the aws port. (e.g. {@value #AWS_BROKER_PORT}=8883).
	 */
	public static final String AWS_BROKER_PORT = AWS + BROKER + "port"; //$NON-NLS-1$

	/**
	 * Property to get the path to the things id. (e.g. {@value #AWS_DEVICE_ID}=myThing).
	 */
	public static final String AWS_DEVICE_ID = AWS + ".device.id"; //$NON-NLS-1$

	private Thing myThing = null;
	private String subscribedTopic;
	private final String host;
	private final String id;
	private final int port;
	private final Properties properties;
	private CertificateLoader certificateLoader;
	private SSLContext sslContext;

	/**
	 * Instantiates a {@link ThermostatAWS} using the System properties and loading the certificates from the
	 * application resources.
	 *
	 * It will require this properties:<br>
	 * - {@value SslContextHelper#AWS_CERTIFICATE_LIST}: a list of certificate to load seperated by coma.<br>
	 * - {@value SslContextHelper#AWS_PRIVATE_KEY_FILE_PASSWORD}: password of the device's private key used.<br>
	 * - {@value SslContextHelper#AWS_PRIVATE_KEY_FILE_PATH}: path to the device's private key file.<br>
	 * - {@value SslContextHelper#AWS_CERTIFICATE_FILE_PATH}: path to the device's certificate file.<br>
	 * - {@value #AWS_BROKER_URL}: path to broker URL.<br>
	 * - {@value #AWS_BROKER_PORT}: port of AWS.<br>
	 * - {@value #AWS_DEVICE_ID}: id of the device.<br>
	 *
	 * @throws IOException
	 *             if a property is not set.
	 */
	public ThermostatAWS() throws IOException {
		this(new Properties(), new ResourceCertificateLoader());
	}

	/**
	 * Instantiates a {@link ThermostatAWS} using the properties and certificate loader provided in parameter.
	 *
	 * It will require this properties:<br>
	 * - {@value SslContextHelper#AWS_CERTIFICATE_LIST}: a list of certificate to load seperated by coma.<br>
	 * - {@value SslContextHelper#AWS_PRIVATE_KEY_FILE_PASSWORD}: password of the device's private key used.<br>
	 * - {@value SslContextHelper#AWS_PRIVATE_KEY_FILE_PATH}: path to the device's private key file.<br>
	 * - {@value SslContextHelper#AWS_CERTIFICATE_FILE_PATH}: path to the device's certificate file.<br>
	 * - {@value #AWS_BROKER_URL}: path to broker URL.<br>
	 * - {@value #AWS_BROKER_PORT}: port of AWS.<br>
	 * - {@value #AWS_DEVICE_ID}: id of the device.<br>
	 *
	 * @param properties
	 *            the properties to use.
	 * @param certificateLoader
	 *            the certificate loader to use.
	 *
	 * @throws IOException
	 *             if a property is not set.
	 */
	public ThermostatAWS(Properties properties, CertificateLoader certificateLoader) throws IOException {
		this.properties = properties;
		this.certificateLoader = certificateLoader;
		this.host = SslContextHelper.getProperty(properties, AWS_BROKER_URL);
		this.port = Integer.parseInt(SslContextHelper.getProperty(properties, AWS_BROKER_PORT));
		this.id = SslContextHelper.getProperty(properties, AWS_DEVICE_ID);
	}

	/**
	 * Instantiates a {@link ThermostatAWS} using the properties and SSLContext provided in parameter.
	 *
	 * It will require this properties:<br>
	 * - {@value #AWS_BROKER_URL}: path to broker URL.<br>
	 * - {@value #AWS_BROKER_PORT}: port of AWS.<br>
	 * - {@value #AWS_DEVICE_ID}: id of the device.<br>
	 *
	 * @param properties
	 *            the properties to use.
	 * @param sslContext
	 *            the ssl context to use.
	 *
	 * @throws IOException
	 *             if a property is not set.
	 */
	public ThermostatAWS(Properties properties, SSLContext sslContext) throws IOException {
		this.properties = properties;
		this.sslContext = sslContext;
		this.host = SslContextHelper.getProperty(properties, AWS_BROKER_URL);
		this.port = Integer.parseInt(SslContextHelper.getProperty(properties, AWS_BROKER_PORT));
		this.id = SslContextHelper.getProperty(properties, AWS_DEVICE_ID);
	}

	@Override
	protected synchronized void doDisconnect() {
		Thing myThing = this.myThing;
		this.myThing = null;
		if (myThing != null) {
			try {
				myThing.disconnect();
			} catch (IotCoreException e) {
				Messages.LOGGER.log(Level.INFO, Messages.CATEGORY, Messages.CLOUD_ERROR, e);
			}
		}
	}

	@Override
	protected synchronized void doConnect() throws IOException {
		if (this.myThing != null) {
			disconnect();
		}
		if (this.myThing == null) {
			try {
				if (this.sslContext == null && this.properties != null && this.certificateLoader != null) {
					this.sslContext = SslContextHelper.getSslContext(this.properties, this.certificateLoader);
				}

				Thing myThing = new Thing(new ConnectOptions().setServerHost(this.host).setServerPort(this.port)
						.setThingId(this.id).setSslContext(this.sslContext));
				myThing.connect();
				this.myThing = myThing;
			} catch (IotCoreException | UnrecoverableKeyException | KeyManagementException | NoSuchAlgorithmException
					| KeyStoreException | CertificateException e) {
				throw new IOException(e);
			}
		}
	}

	@Override
	public String getName() {
		return NAME;
	}

	@Override
	public boolean isConnected() {
		Thing thing = this.myThing;
		return super.isConnected() && thing != null;
	}

	@Override
	protected void subscribe(String topic) throws IOException {
		this.subscribedTopic = topic;
		try {
			this.myThing.addListener(topic, this);
		} catch (IotCoreException e) {
			throw new IOException(e);
		}
	}

	@Override
	public void onDataReceived(String topic, byte[] dataReceived) {
		if (topic.equals(this.subscribedTopic)) {
			receive(new String(dataReceived));
		}
	}

	@Override
	protected void doPublish(String topic, String value) throws IOException {
		Thing myThing = this.myThing;
		if (myThing == null) {
			throw new IOException();
		}
		try {
			myThing.publish(topic, value.getBytes());
		} catch (Exception e) {
			throw new IOException(e);
		}
	}

}
