/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.authz.acl;

import ej.authz.AccessControlEngine;
import ej.basictool.map.PackedMap;

/**
 * An implementation of {@link AccessControlEngine} that uses a user ID, an access mode (read, write, ...) and a Java
 * object as resource to list all permissions per user for each resource controlled in a user-defined database.
 */
public class AccessControlList implements AccessControlEngine<String, String, Object> {

	private final AclDataAccess database;

	/**
	 * Constructs the engine with an empty policy stored in the memory ({@link PackedMap}).
	 *
	 * @see AclInMemoryDatabase
	 */
	public AccessControlList() {
		this(new AclInMemoryDatabase());
	}

	/**
	 * Constructs the engine with the given database.
	 * <p>
	 * To define new resources and permissions, use {@link #addPermission(String, Object, String...)}.
	 *
	 * @param database
	 *            the database that stores the policy (lists of permissions).
	 */
	public AccessControlList(AclDataAccess database) {
		this.database = database;
	}

	/**
	 * Adds a new permission for {@code user} to access {@code resource} with all actions in {@code actions}.
	 *
	 * @param user
	 *            the user to give permissions to.
	 * @param resource
	 *            the resource that is authorized being access to by this permission.
	 * @param actions
	 *            the actions of the request authorized by this permission.
	 */
	public void addPermission(String user, Object resource, String... actions) {
		for (String action : actions) {
			this.database.add(user, action, resource);
		}
	}

	@Override
	public boolean isAuthorized(String user, String action, Object resource) {
		AclEntry entry = this.database.getEntry(user, resource);

		if (entry == null) {
			return false;
		}

		return entry.contains(action);
	}

}
