/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.authz.acl;

import ej.basictool.ArrayTools;

/**
 * List of permissions linked to a user for a specific resource.
 * <p>
 * Uses a growing array of authorized actions.
 *
 * @see ArrayTools
 */
public class AclEntry {

	private String[] actions;

	/**
	 * Constructs an empty ACL entry.
	 */
	public AclEntry() {
		this.actions = new String[0];
	}

	/**
	 * Constructs an ACL entry with given permitted actions.
	 *
	 * @param actions
	 *            the permitted actions.
	 */
	public AclEntry(String... actions) {
		this.actions = actions == null ? new String[0] : ArrayTools.copy(actions);
	}

	/**
	 * Constructs a copy of the ACL entry.
	 *
	 * @param entry
	 *            the ACL entry to copy.
	 */
	public AclEntry(AclEntry entry) {
		this(entry == null ? null : entry.actions);
	}

	/**
	 * Adds an authorized action (read, write, ...).
	 *
	 * @param action
	 *            the action to authorize.
	 */
	public void add(String action) {
		if (!contains(action)) {
			this.actions = ArrayTools.add(this.actions, action);
		}
	}

	/**
	 * Determines if the action is authorized by this entry.
	 *
	 * @param action
	 *            the action requested.
	 * @return {@code true} if the entry contains the {@code action}, {@code false} otherwise.
	 */
	public boolean contains(String action) {
		return ArrayTools.containsEquals(this.actions, action);
	}

}