/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.aws.iot;

import ej.aws.iot.pubsub.Client;
import ej.aws.iot.pubsub.ConnectOptions;
import ej.aws.iot.pubsub.IotCoreException;
import ej.aws.iot.pubsub.TopicListener;

/**
 * This class represents an AWS IoT Thing.
 */
public class Thing {

	private final Client client;

	private final ConnectOptions connectOptions;

	/**
	 * Constructor.
	 *
	 * @param connectOptions
	 *            the options to be configured to be able to connect to the AWS IoT
	 *            platform.
	 */
	public Thing(ConnectOptions connectOptions) {
		this.connectOptions = connectOptions;
		this.client = new Client();
	}

	/**
	 * Connect the Thing to AWS IoT.
	 *
	 * @throws IotCoreException
	 *             when connected
	 */
	public void connect() throws IotCoreException {

		if (isConnected()) {
			throw new IotCoreException("Device already connected.");
		}

		this.client.connect(this.connectOptions);
	}

	/**
	 * Disconnect the Thing from AWS IoT.
	 *
	 * @throws IotCoreException
	 *             when not connected
	 */
	public void disconnect() throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException("Device not connected.");
		}

		this.client.disconnect();
	}

	/**
	 * Check if the thing is connected.
	 *
	 * @return true if connected, false otherwise.
	 */
	public boolean isConnected() {
		return this.client.isConnected();
	}

	/**
	 * Publishes a message on a desired topic.
	 *
	 * @param topic
	 *            the topic to publish to
	 * @param data
	 *            the message
	 * @throws IotCoreException
	 *             if an error occurs
	 */
	public void publish(String topic, byte[] data) throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException("Device not connected.");
		}

		this.client.publish(topic, data);
	}

	/**
	 * Adds a listener on a specified topic.
	 *
	 * @param topic
	 *            the topic to listen to
	 * @param listener
	 *            the listener
	 * @throws IotCoreException
	 *             if an error occurs
	 */
	public void addListener(String topic, TopicListener listener) throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException("Device not connected.");
		}

		this.client.addTopicListener(topic, listener);
	}

}
