/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave.frame;

import java.io.IOException;
import java.io.OutputStream;

import ej.basedriver.zwave.Session;
import ej.basedriver.zwave.ZwaveController;
import ej.basedriver.zwave.ZwaveProtocol;
import ej.basedriver.zwave.frame.RequestFrameListener.RequestFrameError;

public abstract class RequestFrame extends DataFrame implements SendFrame {

	private static byte [] createContent (byte functionID, int functionParameterCount) {
		if (functionParameterCount < 0) {
			throw new IllegalArgumentException();
		}

		byte [] content = new byte [5 + functionParameterCount];
		content[0] = ZwaveProtocol.SOF;
		content[1] = (byte) (content.length - 2);
		content[2] = ZwaveProtocol.REQUEST;
		content[3] = functionID;
		return content;
	}

	private int numberTrasmission;
	protected int parameterIndex;
	protected RequestFrameListener listener;

	/**
	 * @param functionID
	 * @param functionParameterCount
	 * @param listener
	 * @throws IllegalArgumentException
	 */
	public RequestFrame(RequestFrameListener listener, byte functionID, int functionParameterCount) {
		super(createContent(functionID, functionParameterCount));
		this.parameterIndex = 4;
		this.listener = listener;
	}

	/**
	 * @param functionID
	 * @param listener
	 */
	public RequestFrame(RequestFrameListener listener, byte functionID) {
		this(listener, functionID, 0);
	}

	/**
	 * @return the session
	 */
	public abstract Session getSession();

	public void addParameter (byte parameter) {
		if (parameterIndex >= content.length - 1) {
			throw new IndexOutOfBoundsException();
		}

		content[parameterIndex++] = parameter;
	}

	@Override
	public void send(OutputStream os) throws IOException {
		synchronized (os) {
			os.write(content);
			os.flush();
			numberTrasmission ++;
		}
	}

	public void completeContent () {
		content[content.length - 1] = computeChecksum();
	}

	/**
	 * @return
	 */
	public int getNumberRetransmission() {
		return (numberTrasmission == 0) ? 0 : numberTrasmission - 1;
	}

	public void handleRequest (ZwaveController controller, DataFrame dataFrame) throws IOException {
		//do nothing
	}

	public void handleResponse (ZwaveController controller, DataFrame dataFrame) throws IOException {
		Session.RESPONSE.close();
	}

	/**
	 * 
	 */
	public void notifyComplete() {
		if (ZwaveController.DEBUG) {
			System.out.println("Complete " + numberTrasmission);
		}
		if (listener != null) {
			listener.notifyComplete();
		}
	}

	/**
	 * @param error
	 */
	public void notifyError(RequestFrameError error) {
		if (ZwaveController.DEBUG) {
			System.out.println("Error " + error);
		}
		if (listener != null) {
			listener.notifyError(error);
		}
	}

	private static int CALLBACK_COUNTER;
	protected static byte generateCallback () {
		return (byte) (((CALLBACK_COUNTER++ % 255) +1) & 0xFF);
	}
}
