/*
 * Copyright 2013-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.components.registry.util;

import ej.components.dependencyinjection.ServiceLoader;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.components.registry.BundleActivator;
import ej.components.registry.BundleRegistry;

/**
 * Helper class to startup a bundle registry.
 */
public class BundleRegistryHelper {

	private BundleRegistryHelper() {
	}

	/**
	 * Initializes, links and starts the registry.
	 *
	 * @see #getRegistry()
	 */
	public static void startupRegistry() {
		BundleRegistry bundleRegistry = getRegistry();
		if (bundleRegistry != null) {
			startup(bundleRegistry);
		}
	}

	/**
	 * Gets the registry declared as provider of the bundle registry service.
	 *
	 * @return the registry provider singleton.
	 * @see ServiceLoaderFactory#getServiceLoader()
	 * @see ServiceLoader#getService(Class)
	 */
	public static BundleRegistry getRegistry() {
		return ServiceLoaderFactory.getServiceLoader().getService(BundleRegistry.class);
	}

	/**
	 * Initializes, links and starts a bundle.
	 *
	 * @param bundle
	 *            the bundle to initialize.
	 */
	public static void startup(BundleActivator bundle) {
		bundle.initialize();
		bundle.link();
		bundle.start();
	}

	/**
	 * Loads a bundle from a class name.
	 *
	 * @param className
	 *            the name of the class to load.
	 * @return the loaded bundle activator.
	 */
	public static BundleActivator loadBundle(String className) {
		try {
			BundleActivator bundle = (BundleActivator) Class.forName(className).newInstance();
			return bundle;
		} catch (InstantiationException | IllegalAccessException | ClassNotFoundException | ClassCastException e) {
			throw new RuntimeException("Cannot instantiate " + className, e); //$NON-NLS-1$
		}
	}

}
