/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style;

import ej.annotation.NonNull;
import ej.style.background.Background;
import ej.style.border.Border;
import ej.style.dimension.Dimension;
import ej.style.font.FontProfile;
import ej.style.outline.Outline;
import ej.style.text.TextManager;

/**
 * A style describes how elements must be rendered on screen. The attributes of the style are strongly inspired from
 * CSS.
 * <p>
 * The foreground color is used to render the text (or any drawing in the content).
 * <p>
 * The background &amp; the background color are used to render the background of the element.
 * <p>
 * The font profile contains the different properties to select the font used to render the text (or to define the size
 * of a drawn element).
 * <p>
 * The text manager is able to render the text.
 * <p>
 * The alignment defines the position of the content (text or image or other) inside the element bounds.
 * <p>
 * The outlines implement the CSS box model. An outline is a box that wraps another box. An element is wrapped in a
 * sequence of outlines: the padding, the border then the margin:<br>
 * <img src="doc-files/boxmodel.png" alt="Outlines model.">
 * <p>
 * The dimension allows to control the size of an element.
 */
public class Style {

	/**
	 * Foreground color field position in bit map.
	 */
	protected static final int FOREGROUND_COLOR_SHIFT = 0;
	/**
	 * Background color field position in bit map.
	 */
	protected static final int BACKGROUND_COLOR_SHIFT = 1;
	/**
	 * Border color field position in bit map.
	 */
	protected static final int BORDER_COLOR_SHIFT = 2;
	/**
	 * Border field position in bit map.
	 */
	protected static final int BORDER_SHIFT = 3;
	/**
	 * Font profile field position in bit map.
	 */
	protected static final int FONT_PROFILE_SHIFT = 4;
	/**
	 * Alignment field position in bit map.
	 */
	protected static final int ALIGNMENT_SHIFT = 5;
	/**
	 * Text manager field position in bit map.
	 */
	protected static final int TEXT_MANAGER_SHIFT = 6;
	/**
	 * Dimension field position in bit map.
	 */
	protected static final int DIMENSION_SHIFT = 7;
	/**
	 * Padding field position in bit map.
	 */
	protected static final int PADDING_SHIFT = 8;
	/**
	 * Margin field position in bit map.
	 */
	protected static final int MARGIN_SHIFT = 9;
	/**
	 * Background field position in bit map.
	 */
	protected static final int BACKGROUND_SHIFT = 10;
	/**
	 * Map value when all fields are set.
	 */
	protected static final int COMPLETE_MAP = (0x1 << FOREGROUND_COLOR_SHIFT) | (0x1 << BACKGROUND_COLOR_SHIFT)
			| (0x1 << BORDER_COLOR_SHIFT) | (0x1 << BORDER_SHIFT) | (0x1 << FONT_PROFILE_SHIFT)
			| (0x1 << ALIGNMENT_SHIFT) | (0x1 << TEXT_MANAGER_SHIFT) | (0x1 << DIMENSION_SHIFT) | (0x1 << PADDING_SHIFT)
			| (0x1 << MARGIN_SHIFT) | (0x1 << BACKGROUND_SHIFT);
	/**
	 * Bit map for set fields.
	 */
	protected int map;

	/**
	 * Foreground color.
	 */
	protected int foregroundColor;
	/**
	 * Background color.
	 */
	protected int backgroundColor;
	/**
	 * Border color.
	 */
	protected int borderColor;
	/**
	 * Border.
	 */
	protected Border border;
	/**
	 * Font profile.
	 */
	protected FontProfile fontProfile;
	/**
	 * Alignment.
	 */
	protected int alignment;
	/**
	 * Text manager.
	 */
	protected TextManager textManager;
	/**
	 * Dimension.
	 */
	protected Dimension dimension;
	/**
	 * Padding.
	 */
	protected Outline padding;
	/**
	 * Margin.
	 */
	protected Outline margin;
	/**
	 * Background.
	 */
	protected Background background;

	/**
	 * Creates a style.
	 * <ul>
	 * <li>Foreground color is set to <code>0</code></li>
	 * <li>Background color is set to <code>0</code></li>
	 * <li>Border color is set to <code>0</code></li>
	 * <li>Alignment is set to <code>0</code>.</li>
	 * <li>All other attributes are <code>null</code>.</li>
	 * </ul>
	 */
	public Style() {
		// Fields initialization: VM DONE
	}

	/**
	 * Creates a style from another one.
	 *
	 * @param style
	 *            the style to copy from.
	 */
	public Style(@NonNull Style style) {
		this.foregroundColor = style.foregroundColor;
		this.backgroundColor = style.backgroundColor;
		this.borderColor = style.borderColor;
		this.border = style.border;
		this.fontProfile = style.fontProfile;
		this.alignment = style.alignment;
		this.textManager = style.textManager;
		this.dimension = style.dimension;
		this.padding = style.padding;
		this.margin = style.margin;
		this.background = style.background;
		this.map = style.map;
	}

	/**
	 * Gets the foreground color.
	 *
	 * @return the foreground color.
	 */
	public int getForegroundColor() {
		return this.foregroundColor;
	}

	/**
	 * Gets the background color.
	 *
	 * @return the background color.
	 */
	public int getBackgroundColor() {
		return this.backgroundColor;
	}

	/**
	 * Gets the border color.
	 *
	 * @return the border color.
	 */
	public int getBorderColor() {
		return this.borderColor;
	}

	/**
	 * Gets the border.
	 *
	 * @return the border.
	 */
	public Border getBorder() {
		return this.border;
	}

	/**
	 * Gets the font profile.
	 *
	 * @return the font profile.
	 */
	public FontProfile getFontProfile() {
		return this.fontProfile;
	}

	/**
	 * Gets the content alignment.
	 *
	 * @return the content alignment.
	 */
	public int getAlignment() {
		return this.alignment;
	}

	/**
	 * Gets the text manager.
	 *
	 * @return the text manager
	 */
	public TextManager getTextManager() {
		return this.textManager;
	}

	/**
	 * Gets the dimension.
	 *
	 * @return the dimension
	 */
	public Dimension getDimension() {
		return this.dimension;
	}

	/**
	 * Gets the padding.
	 *
	 * @return the padding
	 */
	public Outline getPadding() {
		return this.padding;
	}

	/**
	 * Gets the margin.
	 *
	 * @return the margin.
	 */
	public Outline getMargin() {
		return this.margin;
	}

	/**
	 * Gets the background.
	 *
	 * @return the background.
	 */
	public Background getBackground() {
		return this.background;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Style) {
			Style other = (Style) obj;
			return this.foregroundColor == other.foregroundColor && this.backgroundColor == other.backgroundColor
					&& this.borderColor == other.borderColor && this.alignment == other.alignment
					&& areEqual(this.border, other.border) && areEqual(this.fontProfile, other.fontProfile)
					&& areEqual(this.textManager, other.textManager) && areEqual(this.dimension, other.dimension)
					&& areEqual(this.padding, other.padding) && areEqual(this.margin, other.margin)
					&& areEqual(this.background, other.background);
		}
		return false;
	}

	private static boolean areEqual(Object object1, Object object2) {
		if (object1 == null) {
			return object2 == null;
		} else {
			return object1.equals(object2);
		}
	}

	@Override
	public int hashCode() {
		// Since the fields are not final, they can't be used to compute a constant hashcode.
		return 0;
	}

	/**
	 * Checks whether or not the style is complete. It means that all its attributes are set.
	 *
	 * @return <code>true</code> if the style is complete, <code>false</code> otherwise.
	 */
	public boolean isComplete() {
		return this.map == COMPLETE_MAP;
	}

	/**
	 * Checks whether or not the foreground color of the style is set.
	 *
	 * @return <code>true</code> if the foreground color is set, <code>false</code> otherwise.
	 */
	public boolean isSetForegroundColor() {
		return (this.map & (0x1 << FOREGROUND_COLOR_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the background color of the style is set.
	 *
	 * @return <code>true</code> if the background color is set, <code>false</code> otherwise.
	 */
	public boolean isSetBackgroundColor() {
		return (this.map & (0x1 << BACKGROUND_COLOR_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the border color of the style is set.
	 *
	 * @return <code>true</code> if the border color is set, <code>false</code> otherwise.
	 */
	public boolean isSetBorderColor() {
		return (this.map & (0x1 << BORDER_COLOR_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the border of the style is set.
	 *
	 * @return <code>true</code> if the border is set, <code>false</code> otherwise.
	 */
	public boolean isSetBorder() {
		return (this.map & (0x1 << BORDER_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the font profile of the style is set.
	 *
	 * @return <code>true</code> if the font profile is set, <code>false</code> otherwise.
	 */
	public boolean isSetFontProfile() {
		return (this.map & (0x1 << FONT_PROFILE_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the alignment of the style is set.
	 *
	 * @return <code>true</code> if the alignment is set, <code>false</code> otherwise.
	 */
	public boolean isSetAlignment() {
		return (this.map & (0x1 << ALIGNMENT_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the text manager of the style is set.
	 *
	 * @return <code>true</code> if the text manager is set, <code>false</code> otherwise.
	 */
	public boolean isSetTextManager() {
		return (this.map & (0x1 << TEXT_MANAGER_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the dimension of the style is set.
	 *
	 * @return <code>true</code> if the dimension is set, <code>false</code> otherwise.
	 */
	public boolean isSetDimension() {
		return (this.map & (0x1 << DIMENSION_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the padding of the style is set.
	 *
	 * @return <code>true</code> if the padding is set, <code>false</code> otherwise.
	 */
	public boolean isSetPadding() {
		return (this.map & (0x1 << PADDING_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the margin of the style is set.
	 *
	 * @return <code>true</code> if the margin is set, <code>false</code> otherwise.
	 */
	public boolean isSetMargin() {
		return (this.map & (0x1 << MARGIN_SHIFT)) != 0x0;
	}

	/**
	 * Checks whether or not the background of the style is set.
	 *
	 * @return <code>true</code> if the background is set, <code>false</code> otherwise.
	 */
	public boolean isSetBackground() {
		return (this.map & (0x1 << BACKGROUND_SHIFT)) != 0x0;
	}

	/**
	 * Fills all defined attributes of the given style in this style.
	 *
	 * @param style
	 *            the style to get the attributes from.
	 */
	public void override(@NonNull Style style) {
		// All isSet… methods are inlined for performance purpose.
		int styleMap = style.map;
		if ((styleMap & (0x1 << FOREGROUND_COLOR_SHIFT)) != 0x0) {
			this.foregroundColor = style.foregroundColor;
		}
		if ((styleMap & (0x1 << BACKGROUND_COLOR_SHIFT)) != 0x0) {
			this.backgroundColor = style.backgroundColor;
		}
		if ((styleMap & (0x1 << BORDER_COLOR_SHIFT)) != 0x0) {
			this.borderColor = style.borderColor;
		}
		if ((styleMap & (0x1 << BORDER_SHIFT)) != 0x0) {
			this.border = style.border;
		}
		if ((styleMap & (0x1 << FONT_PROFILE_SHIFT)) != 0x0) {
			this.fontProfile = style.fontProfile;
		}
		if ((styleMap & (0x1 << ALIGNMENT_SHIFT)) != 0x0) {
			this.alignment = style.alignment;
		}
		if ((styleMap & (0x1 << TEXT_MANAGER_SHIFT)) != 0x0) {
			this.textManager = style.textManager;
		}
		if ((styleMap & (0x1 << DIMENSION_SHIFT)) != 0x0) {
			this.dimension = style.dimension;
		}
		if ((styleMap & (0x1 << PADDING_SHIFT)) != 0x0) {
			this.padding = style.padding;
		}
		if ((styleMap & (0x1 << MARGIN_SHIFT)) != 0x0) {
			this.margin = style.margin;
		}
		if ((styleMap & (0x1 << BACKGROUND_SHIFT)) != 0x0) {
			this.background = style.background;
		}
		this.map |= styleMap;
	}

}
