/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.border;

import ej.microui.display.GraphicsContext;
import ej.style.container.Rectangle;

/**
 * Represents and draw a rectangular colored border.
 * <p>
 * Each edge color can be customized.
 */
public class ComplexRectangularColoredBorder extends ComplexRectangularBorder {

	private static final int NO_COLOR = 0xffffffff;

	private int colorTop;
	private int colorBottom;
	private int colorLeft;
	private int colorRight;

	/**
	 * Creates an empty complex rectangular border (0 pixel thickness).
	 */
	public ComplexRectangularColoredBorder() {
		this.colorTop = NO_COLOR;
		this.colorBottom = NO_COLOR;
		this.colorLeft = NO_COLOR;
		this.colorRight = NO_COLOR;
	}

	/**
	 * Creates a complex rectangular border specifying its edges thickness and color.
	 *
	 * @param top
	 *            the top thickness to set.
	 * @param colorTop
	 *            the top color to set.
	 * @param right
	 *            the right thickness to set.
	 * @param colorRight
	 *            the right color to set.
	 * @param bottom
	 *            the bottom thickness to set.
	 * @param colorBottom
	 *            the bottom color to set.
	 * @param left
	 *            the left thickness to set.
	 * @param colorLeft
	 *            the left color to set.
	 */
	public ComplexRectangularColoredBorder(int top, int colorTop, int right, int colorRight, int bottom,
			int colorBottom, int left, int colorLeft) {
		super(top, right, bottom, left);
		this.colorTop = colorTop;
		this.colorRight = colorRight;
		this.colorBottom = colorBottom;
		this.colorLeft = colorLeft;
	}

	/**
	 * Gets the top color.
	 *
	 * @return the top color.
	 */
	public int getColorTop() {
		return this.colorTop;
	}

	/**
	 * Sets the top color.
	 *
	 * @param color
	 *            the color to set.
	 */
	public void setColorTop(int color) {
		this.colorTop = color;
	}

	/**
	 * Sets the top border properties: thickness and color.
	 *
	 * @param thickness
	 *            the thickness to set.
	 * @param color
	 *            the color to set.
	 */
	public void setTop(int thickness, int color) {
		setTop(thickness);
		setColorTop(color);
	}

	/**
	 * Gets the bottom color.
	 *
	 * @return the bottom color.
	 */
	public int getColorBottom() {
		return this.colorBottom;
	}

	/**
	 * Sets the bottom color.
	 *
	 * @param color
	 *            the color to set.
	 */
	public void setColorBottom(int color) {
		this.colorBottom = color;
	}

	/**
	 * Sets the bottom border properties: thickness and color.
	 *
	 * @param thickness
	 *            the thickness to set.
	 * @param color
	 *            the color to set.
	 */
	public void setBottom(int thickness, int color) {
		setBottom(thickness);
		setColorBottom(color);
	}

	/**
	 * Gets the left color.
	 *
	 * @return the left color.
	 */
	public int getColorLeft() {
		return this.colorLeft;
	}

	/**
	 * Sets the left color.
	 *
	 * @param color
	 *            the color to set.
	 */
	public void setColorLeft(int color) {
		this.colorLeft = color;
	}

	/**
	 * Sets the left border properties: thickness and color.
	 *
	 * @param thickness
	 *            the thickness to set.
	 * @param color
	 *            the color to set.
	 */
	public void setLeft(int thickness, int color) {
		setLeft(thickness);
		setColorLeft(color);
	}

	/**
	 * Gets the right color.
	 *
	 * @return the right color.
	 */
	public int getColorRight() {
		return this.colorRight;
	}

	/**
	 * Sets the right color.
	 *
	 * @param color
	 *            the color to set.
	 */
	public void setColorRight(int color) {
		this.colorRight = color;
	}

	/**
	 * Sets the right border properties: thickness and color.
	 *
	 * @param thickness
	 *            the thickness to set.
	 * @param color
	 *            the color to set.
	 */
	public void setRight(int thickness, int color) {
		setRight(thickness);
		setColorRight(color);
	}

	@Override
	public void apply(GraphicsContext g, Rectangle bounds, int color) {
		int width = bounds.getWidth();
		int height = bounds.getHeight();

		// TODO manage corners
		g.setColor(getColor(this.colorTop, color));
		int topThickness = getTop();
		g.fillRect(0, 0, width, topThickness);

		g.setColor(getColor(this.colorBottom, color));
		int bottomThickness = getBottom();
		g.fillRect(0, height - bottomThickness, width, bottomThickness);

		g.setColor(getColor(this.colorLeft, color));
		int leftThickness = getLeft();
		g.fillRect(0, 0, leftThickness, height);

		g.setColor(getColor(this.colorRight, color));
		int rightThickness = getRight();
		g.fillRect(width - rightThickness, 0, rightThickness, height);

		g.translate(getLeft(), getTop());
		unwrap(bounds);
		g.clipRect(0, 0, bounds.getWidth(), bounds.getHeight());
	}

	private static int getColor(int color, int defaultColor) {
		if (color == NO_COLOR) {
			return defaultColor;
		}
		return color;
	}

}
