/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.font.loader;

import java.util.WeakHashMap;

import ej.annotation.NonNull;
import ej.microui.display.Font;
import ej.style.font.FontProfile;
import ej.style.font.filter.CompositeFilter;
import ej.style.font.filter.FontFamilyFilter;
import ej.style.font.filter.FontFilterHelper;
import ej.style.font.filter.FontSizeValueFilter;
import ej.style.font.filter.FontStyleFilter;

/**
 * Abstract implementation of font loader that uses a font family filter and a font size value filter.
 * <p>
 * The font height for a font profile size can be configured by subclasses.
 * <p>
 * It uses a cache to retrieve fonts faster.
 */
public abstract class AbstractFontLoader implements FontLoader {

	private static final String EMPTY_FONT_FAMILY = ""; //$NON-NLS-1$

	private final CompositeFilter<Font> compositeFilter;
	private final FontFamilyFilter fontFamilyFilter;
	private final FontSizeValueFilter fontSizeFilter;
	private final FontStyleFilter fontStyleFilter;

	private final Font[] allFonts;

	private final WeakHashMap<FontProfile, Font> fontsCache;

	/**
	 * Creates the application font loader.
	 */
	public AbstractFontLoader() {
		this.compositeFilter = new CompositeFilter<Font>();

		this.fontFamilyFilter = new FontFamilyFilter(EMPTY_FONT_FAMILY);
		this.compositeFilter.addFilter(this.fontFamilyFilter);

		this.fontSizeFilter = new FontSizeValueFilter(0);
		this.compositeFilter.addFilter(this.fontSizeFilter);

		this.fontStyleFilter = new FontStyleFilter(Font.STYLE_PLAIN);
		this.compositeFilter.addFilter(this.fontStyleFilter);

		this.allFonts = Font.getAllFonts();

		this.fontsCache = new WeakHashMap<>(1);
	}

	@Override
	public Font getFont(FontProfile fontProfile) {
		Font font = this.fontsCache.get(fontProfile);
		if (font == null) {
			font = getFontInternal(fontProfile);
			this.fontsCache.put(fontProfile, font);
		}
		return font;
	}

	private Font getFontInternal(FontProfile fontProfile) {
		this.fontFamilyFilter.setFontFamily(fontProfile.getFamily());

		int sizeValue = getFontHeight(fontProfile);
		this.fontSizeFilter.setSize(sizeValue);

		this.fontStyleFilter.setStyle(fontProfile.getStyle());

		Font[] fonts = FontFilterHelper.filter(this.compositeFilter, this.allFonts);
		if (fonts.length >= 1) {
			@SuppressWarnings("null")
			@NonNull
			Font font = fonts[0];
			return font;
		}

		// No font matches with the font profile.
		@SuppressWarnings("null")
		@NonNull
		Font defaultFont = Font.getDefaultFont();
		return defaultFont;
	}

	/**
	 * Gets the expected font height in pixels for the given font profile.
	 *
	 * @param fontProfile
	 *            the font profile.
	 * @return the font height.
	 */
	protected abstract int getFontHeight(FontProfile fontProfile);

}
