/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.basic;

import java.util.StringTokenizer;

import ej.annotation.Nullable;
import ej.basictool.ArrayTools;
import ej.bon.Timer;
import ej.bon.TimerTask;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.giml.annotation.Element;
import ej.giml.annotation.ElementConstructor;
import ej.giml.annotation.ElementRequiredAttribute;
import ej.microui.display.GraphicsContext;
import ej.microui.display.Image;
import ej.mwt.MWT;
import ej.style.Style;
import ej.style.container.AlignmentHelper;
import ej.style.container.Rectangle;
import ej.style.image.ImageLoader;
import ej.style.util.StyleHelper;
import ej.widget.StyledWidget;

/**
 * A widget that displays a sequence of images at a fixed rate.
 *
 * @see ej.microui.display.Image
 * @see ImageLoader
 */
@Element(friendlyName = "Animated Image")
public class AnimatedImage extends StyledWidget {

	private static final String SPACE = " "; //$NON-NLS-1$

	private String[] frames;
	private long period;

	private int currentIndex;

	@Nullable
	private TimerTask timerTask;

	/**
	 * Creates an animated image with the path to the images to display.
	 * <p>
	 * The image loader service is used to load the images from the given paths.
	 *
	 * @param frames
	 *            the paths to the frames to display.
	 * @param period
	 *            the period between each image.
	 * @throws NullPointerException
	 *             if the given frames array is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given frames array is empty.
	 * @throws IllegalArgumentException
	 *             if the given period is lower or equal to zero.
	 * @see ImageLoader#getImage(String)
	 */
	public AnimatedImage(String[] frames, long period) {
		setEnabled(false);
		if (frames.length == 0) {
			throw new IllegalArgumentException();
		}
		String[] framesCopy = ArrayTools.copy(frames);
		this.frames = framesCopy;
		this.currentIndex = 0;
		setPeriod(period);
	}

	/**
	 * Creates an animated image with the path to the images to display.
	 * <p>
	 * The image loader service is used to load the images from the given paths.
	 * <p>
	 * The given sources string is a space separated list of paths.
	 *
	 * @param frames
	 *            the paths to the frames to display.
	 * @param period
	 *            the period between each image.
	 * @throws NullPointerException
	 *             if the given frames string is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given frames string is empty.
	 * @throws IllegalArgumentException
	 *             if the given period is lower or equal to zero.
	 * @see ImageLoader#getImage(String)
	 */
	@ElementConstructor
	public AnimatedImage(@ElementRequiredAttribute String frames, @ElementRequiredAttribute long period) {
		setEnabled(false);
		StringTokenizer tokenizer = new StringTokenizer(frames, SPACE);
		int length = tokenizer.countTokens();
		String[] framesArray = new String[length];
		int currentIndex = 0;
		while (tokenizer.hasMoreTokens()) {
			framesArray[currentIndex++] = tokenizer.nextToken();
		}
		if (currentIndex == 0) {
			throw new IllegalArgumentException();
		}
		this.frames = framesArray;
		this.currentIndex = 0;
		setPeriod(period);
	}

	/**
	 * Sets the period.
	 *
	 * @param period
	 *            the period to set.
	 * @throws IllegalArgumentException
	 *             if the given period is lower or equal to zero.
	 */
	public void setPeriod(long period) {
		if (period <= 0) {
			throw new IllegalArgumentException();
		}
		this.period = period;
	}

	/**
	 * Gets the period.
	 *
	 * @return the period.
	 */
	public long getPeriod() {
		return this.period;
	}

	/**
	 * Sets the frames to display for this animated image.
	 *
	 * @param frames
	 *            the path to the frames to display.
	 * @throws NullPointerException
	 *             if the given frames array is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given frames array is empty.
	 * @see ImageLoader#getImage(String)
	 */
	public void setFrames(String[] frames) {
		// Copy array to avoid external modifications.
		String[] framesCopy = ArrayTools.copy(frames);
		setFramesInternal(framesCopy);
	}

	/**
	 * Sets the frames. The given array is stored directly. Make sure the array is referenced only by this instance.
	 *
	 * @param frames
	 *            the path to the frames to display.
	 */
	private void setFramesInternal(String[] frames) { // NOSONAR see javadoc.
		if (frames.length == 0) {
			throw new IllegalArgumentException();
		}
		this.frames = frames;
		this.currentIndex = 0;
		invalidate();
	}

	/**
	 * Gets the frames.
	 *
	 * @return the frames.
	 */
	public String[] getFrames() {
		return ArrayTools.copy(this.frames);
	}

	@Override
	public void showNotify() {
		super.showNotify();
		Timer timer = ServiceLoaderFactory.getServiceLoader().getService(Timer.class, Timer.class);
		this.timerTask = new TimerTask() {
			@Override
			public void run() {
				next();
			}
		};
		timer.schedule(this.timerTask, this.period, this.period);
	}

	@Override
	public void hideNotify() {
		super.hideNotify();
		TimerTask timerTask = this.timerTask;
		if (timerTask != null) {
			timerTask.cancel();
		}
	}

	private void next() {
		if (this.currentIndex == this.frames.length - 1) {
			this.currentIndex = 0;
		} else {
			this.currentIndex++;
		}
		repaint();
	}

	@Override
	public void renderContent(GraphicsContext g, Style style, Rectangle availableBounds) {
		if (this.frames != null) {
			ej.microui.display.Image frame = StyleHelper.getImage(this.frames[this.currentIndex]);
			if (frame != null) {
				int alignment = style.getAlignment();
				int x = AlignmentHelper.computeXLeftCorner(frame.getWidth(), 0, availableBounds.getWidth(), alignment);
				int y = AlignmentHelper.computeYTopCorner(frame.getHeight(), 0, availableBounds.getHeight(), alignment);
				g.setColor(style.getForegroundColor());
				g.drawImage(frame, x, y, GraphicsContext.LEFT | GraphicsContext.TOP);
			}
		}
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle availableBounds) {
		Rectangle contentSize = new Rectangle();

		String[] frames = this.frames;
		if (frames != null) {
			Image firstImage = StyleHelper.getImage(frames[0]);
			if (firstImage != null) {
				int width = getSize(availableBounds.getWidth(), firstImage.getWidth());
				int height = getSize(availableBounds.getHeight(), firstImage.getHeight());
				contentSize.setSize(width, height);
			}
		}

		return contentSize;
	}

	private int getSize(int available, int preferred) {
		return available == MWT.NONE ? preferred : Math.min(available, preferred);
	}

}
