/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.basic;

import java.util.Arrays;

import ej.annotation.Nullable;
import ej.bon.Timer;
import ej.bon.TimerTask;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.giml.annotation.Element;
import ej.giml.annotation.ElementAttribute;
import ej.giml.annotation.ElementConstructor;

/**
 * Password is a text with hidden characters.
 * <p>
 * The {@link #getText()} method is the text displayed. It returns a string that replaces each character of the password
 * by the character '\u25cf'. The value of the password can be retrieved using {@link #getPassword()}.
 */
@Element
public class Password extends Text {

	private static final String EMPTY_STRING = ""; //$NON-NLS-1$

	private static final char HIDING_CHARACTER = '\u25cf';
	private static final long HIDE_NEW_CHARACTER_DELAY = 500;

	private boolean hasNewCharacter;
	private boolean reveal;
	private char newCharacter;
	@Nullable
	private TimerTask hideNewCharacterTask;

	/**
	 * Creates an empty password.
	 */
	public Password() {
		super();
	}

	/**
	 * Creates a password with an initial content.
	 *
	 * @param password
	 *            the password to set.
	 * @throws NullPointerException
	 *             if the given text is <code>null</code>.
	 */
	public Password(String password) {
		super(password);
	}

	/**
	 * Creates a password with an initial content and a place holder.
	 * <p>
	 * The place holder is displayed when the password is empty.
	 *
	 * @param password
	 *            the password to set.
	 * @param placeHolder
	 *            the place holder to set.
	 * @throws NullPointerException
	 *             if one or both the given text and place holder are <code>null</code>.
	 */
	@ElementConstructor
	public Password(@ElementAttribute(defaultValue = EMPTY_STRING) String password,
			@ElementAttribute(defaultValue = EMPTY_STRING) String placeHolder) {
		super(password, placeHolder);
	}

	@Override
	public String getText() {
		if (this.reveal) {
			return super.getText();
		} else if (this.hasNewCharacter) {
			char[] characters = new char[getTextLength() - 1];
			Arrays.fill(characters, HIDING_CHARACTER);
			return new StringBuilder().append(characters).append(this.newCharacter).toString();
		} else {
			char[] characters = new char[getTextLength()];
			Arrays.fill(characters, HIDING_CHARACTER);
			return new String(characters);
		}
	}

	/**
	 * Gets whether or not the password is revealed.
	 *
	 * @return <code>true</code> if the password is revealed, <code>false</code> otherwise.
	 */
	public boolean isRevealed() {
		return this.reveal;
	}

	/**
	 * Reveals or hides the password.
	 *
	 * @param reveal
	 *            <code>true</code> to reveal the password and <code>false</code> to hide it.
	 */
	public void reveal(boolean reveal) {
		this.reveal = reveal;
		repaint();
	}

	/**
	 * Gets the text of the password not hidden.
	 *
	 * @return the text of the password.
	 */
	public String getPassword() {
		return super.getText();
	}

	@Override
	public void insert(char c) {
		this.hasNewCharacter = true;
		this.newCharacter = c;
		hideNewCharacterAfterDelay();
		super.insert(c);
	}

	private void hideNewCharacterAfterDelay() {
		cancelHideTask();

		Timer timer = ServiceLoaderFactory.getServiceLoader().getService(Timer.class, Timer.class);
		this.hideNewCharacterTask = new TimerTask() {
			@Override
			public void run() {
				hideLastCharacter();
			}
		};
		timer.schedule(this.hideNewCharacterTask, HIDE_NEW_CHARACTER_DELAY);
	}

	private void hideLastCharacter() {
		this.hasNewCharacter = false;
		repaint();
	}

	@Override
	public void hideNotify() {
		super.hideNotify();
		cancelHideTask();
	}

	private void cancelHideTask() {
		TimerTask hideNewCharacterTask = this.hideNewCharacterTask;
		if (hideNewCharacterTask != null) {
			hideNewCharacterTask.cancel();
		}
	}
}