/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.composed;

import ej.giml.annotation.Element;
import ej.giml.annotation.ElementChild;
import ej.giml.annotation.ElementConstructor;
import ej.giml.annotation.ElementRequiredAttribute;
import ej.mwt.Widget;
import ej.widget.basic.Label;

/**
 * A facility to use a button that displays a text.
 * <p>
 * It is simply a button composite that contains a label.
 * <p>
 * This example shows a simple button:
 *
 * <pre>
 * Button button = new Button("Press Me");
 * </pre>
 *
 * <img src="../basic/doc-files/button-simple.png" alt="Simple button.">
 * <p>
 * This example shows a styled button (be aware that the label is drawn over the button, so its background must be
 * transparent in order to see the button background):
 *
 * <pre>
 * Button button = new Button("Press Me");
 *
 * EditableStyle buttonStyle = new EditableStyle();
 * buttonStyle.setBackgroundColor(Colors.SILVER);
 * buttonStyle.setBorder(new SimpleRectangularBorder(1));
 * buttonStyle.setFontProfile(new FontProfile("", 23, Font.STYLE_PLAIN));
 * button.mergeStyle(buttonStyle);
 *
 * EditableStyle buttonLabelStyle = new EditableStyle();
 * buttonLabelStyle.setBackground(NoBackground.NO_BACKGROUND);
 * button.getLabel().mergeStyle(buttonLabelStyle);
 * </pre>
 *
 * <img src="../basic/doc-files/button-styled.png" alt="Styled button.">
 * <p>
 * This example shows a styled button with its background color changing on a press action:
 *
 * <pre>
 * Button button = new Button("Press Me");
 *
 * Stylesheet stylesheet = StyleHelper.getStylesheet();
 *
 * EditableStyle buttonStyle = new EditableStyle();
 * buttonStyle.setBackgroundColor(Colors.SILVER);
 * buttonStyle.setBorder(new SimpleRectangularBorder(1));
 * buttonStyle.setFontProfile(new FontProfile("", 23, Font.STYLE_PLAIN));
 * stylesheet.addRule(new TypeSelector(Button.class), buttonStyle);
 *
 * EditableStyle pressedButtonStyle = new EditableStyle();
 * pressedButtonStyle.setBackgroundColor(Colors.GRAY);
 * stylesheet.addRule(new AndCombinator(new TypeSelector(Button.class), new StateSelector(State.Active)),
 * 		pressedButtonStyle);
 *
 * EditableStyle buttonLabelStyle = new EditableStyle();
 * buttonLabelStyle.setBackground(NoBackground.NO_BACKGROUND);
 * stylesheet.addRule(new ChildCombinator(new TypeSelector(Button.class), new TypeSelector(Label.class)),
 * 		buttonLabelStyle);
 * </pre>
 *
 * <img src="../basic/doc-files/button-styled-pressed.png" alt="Styled pressed button.">
 *
 * @see Label
 * @deprecated see {@link ej.widget.basic.Button}
 */
@Deprecated
@Element
public class Button extends ButtonWrapper {

	private final Label label;

	/**
	 * Creates a button with an empty text.
	 */
	public Button() {
		this.label = new Label();
		super.setWidget(this.label);
	}

	/**
	 * Creates a button with the given text to display. The text cannot be <code>null</code>.
	 *
	 * @param text
	 *            the text to display.
	 * @throws NullPointerException
	 *             if the text is <code>null</code>.
	 */
	@ElementConstructor
	public Button(@ElementRequiredAttribute String text) {
		this.label = new Label(text);
		super.setWidget(this.label);
	}

	@Override
	@ElementChild(prefix = "setWidget", disabled = true)
	public void setWidget(Widget widget) {
		throw new IllegalArgumentException();
	}

	/**
	 * Gets the label contained in the button.
	 *
	 * @return the label contained in the button.
	 */
	public Label getLabel() {
		return this.label;
	}

	/**
	 * Sets the text displayed by this button.
	 *
	 * @param text
	 *            the text to display.
	 * @throws NullPointerException
	 *             if the text is <code>null</code>.
	 * @see Label#setText(String)
	 */
	public void setText(String text) {
		this.label.setText(text);
	}

	/**
	 * Gets the text displayed by this button.
	 * <p>
	 * The text is hold by the inner label.
	 *
	 * @return the text.
	 * @see Label#getText()
	 */
	public String getText() {
		return this.label.getText();
	}

}
