/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.composed;

import ej.giml.annotation.Element;
import ej.giml.annotation.ElementConstructor;
import ej.mwt.Panel;
import ej.style.State;
import ej.widget.listener.OnClickListener;
import ej.widget.util.ButtonHelper;
import ej.widget.util.GenericListener;

/**
 * A button wrapper is a composite that reacts to click events.
 */
@Element(friendlyName = "Button Wrapper")
public class ButtonWrapper extends Wrapper {

	private final ButtonHelper buttonHelper;

	/**
	 * Creates a button without listener.
	 */
	@ElementConstructor
	public ButtonWrapper() {
		this.buttonHelper = new ButtonHelper(new GenericListener() {
			@Override
			public void update() {
				updateStyle();
			}
		});
		this.buttonHelper.addOnClickListener(new OnClickListener() {
			@Override
			public void onClick() {
				ButtonWrapper.this.onClick();
			}
		});
	}

	/**
	 * Invoked when the button has been clicked.
	 */
	protected void onClick() {
		// Do nothing by default.
	}

	/**
	 * Performs the actions associated to a click.
	 */
	public void performClick() {
		this.buttonHelper.performClick();
	}

	/**
	 * Adds a listener on the click events of the button.
	 *
	 * @param listener
	 *            the listener to add.
	 * @throws NullPointerException
	 *             if the given listener is <code>null</code>.
	 */
	public void addOnClickListener(OnClickListener listener) {
		this.buttonHelper.addOnClickListener(listener);
	}

	/**
	 * Removes a listener on the click events of the button.
	 *
	 * @param listener
	 *            the listener to remove.
	 */
	public void removeOnClickListener(OnClickListener listener) {
		this.buttonHelper.removeOnClickListener(listener);
	}

	@Override
	public boolean isInState(State state) {
		return (state == State.Active && this.buttonHelper.isPressed()) || super.isInState(state);
	}

	@Override
	public boolean handleEvent(int event) {
		if (this.buttonHelper.handleEvent(event)) {
			return true;
		}
		return super.handleEvent(event);
	}

	@Override
	public void requestFocus() {
		Panel panel = getPanel();
		if (panel != null) {
			panel.setFocus(this);
		}
	}

	@Override
	public boolean requestFocus(int direction) throws IllegalArgumentException {
		if (hasFocus()) {
			return false;
		} else {
			requestFocus();
			return true;
		}
	}

}
