/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.container;

import ej.annotation.Nullable;
import ej.giml.annotation.Element;
import ej.giml.annotation.ElementAttribute;
import ej.giml.annotation.ElementChild;
import ej.giml.annotation.ElementConstructor;
import ej.mwt.MWT;
import ej.mwt.Widget;
import ej.style.Style;
import ej.style.container.Rectangle;
import ej.widget.StyledComposite;

/**
 * Lays out its children in two regions, giving each one a ratio of its size.
 * <p>
 * Horizontal:
 * <p>
 * <img src="doc-files/split_horizontal.png" alt="Horizontal split.">
 * <p>
 * Vertical:
 * <p>
 * <img src="doc-files/split_vertical.png" alt="Vertical split.">
 */
@Element
public class Split extends StyledComposite {

	private static final float DEFAULT_RATIO = 0.5f;

	private boolean horizontal;
	private float ratio;
	@Nullable
	private Widget first;
	@Nullable
	private Widget last;

	/**
	 * Creates a horizontal split composite, each widget filling half the size.
	 */
	public Split() {
		this.horizontal = true;
		this.ratio = DEFAULT_RATIO;
	}

	/**
	 * Creates a split composite specifying its orientation and the fill ratio of the first widget. Second widget will
	 * be given the remaining space.
	 *
	 * @param horizontal
	 *            <code>true</code> to set the composite horizontal, <code>false</code> to set the composite vertical.
	 * @param ratio
	 *            the fill ratio to set for the first widget.
	 * @throws IllegalArgumentException
	 *             if the given ratio is not between <code>0.0</code> and <code>1.0</code> excluded.
	 * @see #setRatio(float)
	 */
	@ElementConstructor
	public Split(@ElementAttribute(defaultValue = "true") boolean horizontal,
			@ElementAttribute(defaultValue = "0.5f") float ratio) {
		this.horizontal = horizontal;
		setRatioInternal(ratio);
	}

	/**
	 * Sets the composite orientation: horizontal or vertical.
	 *
	 * @param horizontal
	 *            <code>true</code> to set the composite horizontal, <code>false</code> to set the composite vertical.
	 */
	public void setHorizontal(boolean horizontal) {
		this.horizontal = horizontal;
		invalidate();
	}

	/**
	 * Gets whether the composite is horizontal or not.
	 *
	 * @return <code>true</code> if the composite is horizontal, <code>false</code> otherwise.
	 */
	public boolean isHorizontal() {
		return this.horizontal;
	}

	/**
	 * Sets the position of the separation between the children, that is the first child size relatively to this
	 * composite size.
	 * <p>
	 * Example: setting the ratio to <tt>1/3</tt> in horizontal mode will give a third of the width to the first child,
	 * and two third for the last one. Both will be given all available height.
	 *
	 * @param ratio
	 *            the ratio to set.
	 * @throws IllegalArgumentException
	 *             if the given ratio is not between <code>0.0</code> and <code>1.0</code> excluded.
	 */
	public void setRatio(float ratio) {
		setRatioInternal(ratio);
		invalidate();
	}

	/**
	 * Gets the ratio.
	 *
	 * @return the ratio.
	 * @see #setRatio(float)
	 */
	public float getRatio() {
		return this.ratio;
	}

	/**
	 * Sets the ratio.
	 *
	 * @param ratio
	 *            the ratio to set.
	 * @throws IllegalArgumentException
	 *             if the given ratio is not between <code>0.0</code> and <code>1.0</code> excluded.
	 */
	private void setRatioInternal(float ratio) {
		if (ratio <= 0.0f || ratio >= 1.0f) {
			throw new IllegalArgumentException();
		}
		this.ratio = ratio;
	}

	/**
	 * Sets the first widget of this composite. If the split is horizontal, the first widget is on the left side,
	 * otherwise it is on the upper side.
	 *
	 * @param widget
	 *            the widget to add.
	 * @throws NullPointerException
	 *             if the given widget is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given widget or one of its children is already connected to a panel.
	 * @see #add(Widget)
	 */
	@ElementChild
	public void setFirst(Widget widget) {
		Widget firstLocal = this.first; // avoid synchronizing
		if (firstLocal != null) {
			remove(firstLocal);
		}
		this.first = widget;
		add(widget);
	}

	/**
	 * Gets the first widget of this composite.
	 *
	 * @return the first widget of this composite.
	 */
	@Nullable
	public Widget getFirst() {
		return this.first;
	}

	/**
	 * Sets the last widget of this composite. If the split is horizontal, the last widget is on the right side,
	 * otherwise it is on the lower side.
	 *
	 * @param widget
	 *            the widget to add.
	 * @throws NullPointerException
	 *             if the given widget is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given widget or one of its children is already connected to a panel.
	 * @see #add(Widget)
	 */
	@ElementChild
	public void setLast(Widget widget) {
		Widget lastLocal = this.last; // avoid synchronizing
		if (lastLocal != null) {
			remove(lastLocal);
		}
		this.last = widget;
		add(widget);
	}

	/**
	 * Gets the last widget of this composite.
	 *
	 * @return the last widget of this composite.
	 */
	@Nullable
	public Widget getLast() {
		return this.last;
	}

	@Override
	public void remove(Widget widget) throws NullPointerException {
		if (widget == this.first) {
			this.first = null;
		} else if (widget == this.last) {
			this.last = null;
		}
		super.remove(widget);
	}

	@Override
	public void removeAllWidgets() {
		this.first = null;
		this.last = null;
		super.removeAllWidgets();
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle bounds) {
		int widthHint = bounds.getWidth();
		int heightHint = bounds.getHeight();

		if (getWidgetsCount() > 0) {
			float ratio = this.ratio; // avoid synchronizing
			boolean computeWidth = widthHint == MWT.NONE;
			boolean computeHeight = heightHint == MWT.NONE;

			// compute widgets expected width
			int firstWidth;
			int lastWidth;
			if (computeWidth) {
				firstWidth = MWT.NONE;
				lastWidth = MWT.NONE;
			} else {
				if (this.horizontal) {
					firstWidth = (int) (widthHint * ratio);
					lastWidth = widthHint - firstWidth;
				} else {
					firstWidth = widthHint;
					lastWidth = widthHint;
				}
			}
			// compute widgets expected height
			int firstHeight;
			int lastHeight;
			if (computeHeight) {
				firstHeight = MWT.NONE;
				lastHeight = MWT.NONE;
			} else {
				if (this.horizontal) {
					firstHeight = heightHint;
					lastHeight = heightHint;
				} else {
					firstHeight = (int) (heightHint * ratio);
					lastHeight = heightHint - firstHeight;
				}
			}

			// validate widgets and get their preferred widgets
			Widget first = this.first;
			int firstPreferredWidth;
			int firstPreferredHeight;
			if (first != null) {
				first.validate(firstWidth, firstHeight);
				firstPreferredWidth = first.getPreferredWidth();
				firstPreferredHeight = first.getPreferredHeight();
			} else {
				firstPreferredWidth = 0;
				firstPreferredHeight = 0;
			}
			int lastPreferredWidth;
			int lastPreferredHeight;
			Widget last = this.last;
			if (last != null) {
				last.validate(lastWidth, lastHeight);
				lastPreferredWidth = last.getPreferredWidth();
				lastPreferredHeight = last.getPreferredHeight();
			} else {
				lastPreferredWidth = 0;
				lastPreferredHeight = 0;
			}

			// compute composite preferred size if necessary
			widthHint = updatePreferredSize(true, widthHint, computeWidth, firstPreferredWidth, lastPreferredWidth,
					ratio);
			heightHint = updatePreferredSize(false, heightHint, computeHeight, firstPreferredHeight,
					lastPreferredHeight, ratio);
		}

		return new Rectangle(0, 0, widthHint, heightHint);
	}

	private int updatePreferredSize(boolean width, int sizeHint, boolean computeSize, int firstPreferredSize,
			int lastPreferredSize, float ratio) {
		if (computeSize) {
			if (width == this.horizontal) {
				float firstU = firstPreferredSize / ratio;
				float lastU = lastPreferredSize / (1f - ratio);
				if (firstU > lastU) {
					return (int) firstU;
				} else {
					return (int) lastU;
				}
			} else {
				return Math.max(firstPreferredSize, lastPreferredSize);
			}
		}
		return sizeHint;
	}

	@Override
	protected void setBoundsContent(Rectangle bounds) {
		int x = bounds.getX();
		int y = bounds.getY();
		int width = bounds.getWidth();
		int height = bounds.getHeight();

		if (getWidgetsCount() > 0) {
			// compute widgets bounds
			int firstX = x;
			int firstY = y;
			int firstWidth;
			int firstHeight;
			int lastX;
			int lastY;
			int lastWidth;
			int lastHeight;
			if (this.horizontal) {
				firstWidth = (int) (width * this.ratio);
				firstHeight = height;
				lastX = firstWidth;
				lastY = y;
				lastWidth = width - firstWidth;
				lastHeight = height;
			} else {
				firstWidth = width;
				firstHeight = (int) (height * this.ratio);
				lastX = x;
				lastY = firstHeight;
				lastWidth = width;
				lastHeight = height - firstHeight;
			}

			// set widgets bounds
			Widget first = this.first;
			if (first != null) {
				first.setBounds(firstX, firstY, firstWidth, firstHeight);
			}
			Widget last = this.last;
			if (last != null) {
				last.setBounds(lastX, lastY, lastWidth, lastHeight);
			}
		}
	}

}
